local StoneGate = ...

-- Apply building asset processor
StoneGate:registerAssetProcessor("models/basic/basicSmallCircularTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
StoneGate:registerAssetProcessor("models/roofs/simpleRoofs/roofSmallCircularTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
StoneGate:registerAssetProcessor("models/roofs/domeRoofs/domeSmallCircularTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
StoneGate:registerAssetProcessor("models/ramparts/rampartSmallCircularTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register Small Circular Tower materials
StoneGate:registerAssetId("models/roofs/simpleRoofs/roofSmallCircularTower.fbx/Materials/redRoof", StoneGateSetPrefix .. "_SCT_RED_TILE_ROOF_MATERIAL")
StoneGate:registerAssetId("models/roofs/domeRoofs/domeSmallCircularTower.fbx/Materials/redRoof", StoneGateSetPrefix .. "_SCT_DOME_RED_TILE_ROOF_MATERIAL")
StoneGate:registerAssetId("models/basic/basicSmallCircularTower.fbx/Materials/murChateau", StoneGateSetPrefix .. "_SCT_STONE_MANOR_WALL_MATERIAL")
StoneGate:registerAssetId("models/ramparts/rampartSmallCircularTower.fbx/Materials/blueCrenellation", StoneGateSetPrefix .. "_RAMPART_SCT_STONE_MANOR_CRENELLATION_MATERIAL")
StoneGate:registerAssetId("models/basic/basicSmallCircularTower.fbx/Materials/whiteStoneGround", StoneGateSetPrefix .. "_SCT_WHITE_STONE_FLOOR_GROUND_MATERIAL")
StoneGate:registerAssetId("models/basic/basicSmallCircularTower.fbx/Materials/whiteStoneFrame", StoneGateSetPrefix .. "_SCT_WHITE_STONE_FRAME_MATERIAL")

-- Register Small Circular Tower prefab nodes
StoneGate:registerAssetId("models/basic/basicSmallCircularTower.fbx/Prefab/smallCircularTower/step_3_Sct_TopPart", StoneGateSetPrefix .. "_SCT_TOP_PREFAB")

--------------------------------------------------------------------------
-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	StoneGate:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
                IsVisibleWhenBuilt = false, -- to mask the green diamond in top, only visible in preview
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 1*_NoSandbox }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		    }
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.1 }
	        }
	})
end

registerDefaultBuildingPart(StoneGateSetPrefix .. "_SCT_TOP")

function registerSmallCircularTower(_option)
    -- Register Small Circular Tower prefab nodes
    StoneGate:registerAssetId("models/basic/basicSmallCircularTower.fbx/Prefab/smallCircularTower/step_0_Sct_Root" .. _option .. "Part", StoneGateSetPrefix .. "_SCT_ROOT" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicSmallCircularTower.fbx/Prefab/smallCircularTower/step_1_Sct_Tiling1" .. _option .. "Part", StoneGateSetPrefix .. "_SCT_TILING1" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicSmallCircularTower.fbx/Prefab/smallCircularTower/step_2_Sct_Tiling2" .. _option .. "Part", StoneGateSetPrefix .. "_SCT_TILING2" .. _option .. "_PREFAB")
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_SCT_TILING1" .. _option)
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_SCT_TILING2" .. _option)

    -- Register the stone gate set small circular tower root part
    StoneGate:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SCT_ROOT" .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_SCT_ROOT" .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_SCT_ROOT" .. _option .. "_DESC",
	Category = "CORE",
        BuildingZone = _SmallCircularBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_SCT_ROOT" .. _option .. "_PREFAB",
		EndPart = StoneGateSetPrefix .. "_SCT_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_SCT_TILING2" .. _option .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_SCT_TILING1" .. _option .. "_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	},
        VillagerCapacity = 1,
	AssetBuildingFunction = registerAssignableFunction(StoneGateSetPrefix, "SCT_ROOT" .. _option)
    })

    -- Register grass clearing components
    StoneGate:registerPrefabComponent("models/basic/basicSmallCircularTower.fbx/Prefab/smallCircularTower/step_0_Sct_Root" .. _option .. "Part", {
	DataType = "COMP_DIRT_CIRCLE",
	HardRadius = 1,
        SoftRadius = 1,
        IsPermanent = true
    })
end

registerSmallCircularTower("") -- with stone ground
registerSmallCircularTower("P") -- with wooden ground
registerSmallCircularTower("HALFWHITE") -- half-timbered

-- Roof ...
 
function registerSctDefaultMajorPart(_nameFbxFile, _nameFbx, _nameAsset, _resource)
    StoneGate:registerAssetId("models/" .. _nameFbxFile .. "/Prefab/smallCircularTower/" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_SCT_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration roof part
    StoneGate:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SCT_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_SCT_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_SCT_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_SCT_" .. _nameAsset .. "_PREFAB",
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = _resource, Quantity = 2*_NoSandbox }
		}
	}
    })
end

registerSctDefaultMajorPart( "roofs/simpleRoofs/roofSmallCircularTower.fbx", "SmallCircularRoof45", "ROOF45", "PLANK")
registerSctDefaultMajorPart( "roofs/simpleRoofs/roofSmallCircularTower.fbx", "SmallCircularRoof30", "ROOF30", "PLANK")
registerSctDefaultMajorPart( "roofs/simpleRoofs/roofSmallCircularTower.fbx", "SmallCircularRoof60", "ROOF60", "PLANK")
registerSctDefaultMajorPart( "roofs/domeRoofs/domeSmallCircularTower.fbx", "SmallCircularRoofDome", "ROOF_DOME", "PLANK" )
registerSctDefaultMajorPart( "basic/basicSmallCircularTower.fbx", "SmallCircularStoneFloor", "STONE_FLOOR", "STONE" )
registerSctDefaultMajorPart( "basic/basicSmallCircularTower.fbx", "SmallCircularStoneFloorP", "STONE_FLOORP", "STONE" )
registerSctDefaultMajorPart( "basic/basicSmallCircularTower.fbx", "SmallCircularStoneFloorBase", "STONE_FLOOR_BASE", "STONE" )
registerSctDefaultMajorPart( "basic/basicSmallCircularTower.fbx", "SmallCircularStoneFloorBaseP", "STONE_FLOOR_BASEP", "STONE" )
registerSctDefaultMajorPart( "ramparts/rampartSmallCircularTower.fbx", "step_4_Sct_RampartClosed", "RAMPART_CLOSED", "STONE" )
registerSctDefaultMajorPart( "ramparts/rampartSmallCircularTower.fbx", "step_4_Sct_RampartHalf", "RAMPART_HALF", "STONE" )
