local StoneGate = ...

-- Apply building asset processor
StoneGate:registerAssetProcessor("models/other/stoneDecorations.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
StoneGate:registerAssetProcessor("models/other/woodenDecorations.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register Small Squared Tower materials
StoneGate:registerAssetId("models/other/stoneDecorations.fbx/Materials/blueCrenellation", StoneGateSetPrefix .. "_OTHER_STONE_MANOR_CRENELLATION_MATERIAL")
StoneGate:registerAssetId("models/other/stoneDecorations.fbx/Materials/murChateau", StoneGateSetPrefix .. "_OTHER_STONE_MANOR_WALL_MATERIAL")
StoneGate:registerAssetId("models/other/stoneDecorations.fbx/Materials/whiteStoneGround", StoneGateSetPrefix .. "_OTHER_WHITE_STONE_FLOOR_GROUND_MATERIAL")

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	StoneGate:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
                IsVisibleWhenBuilt = false, -- to mask the green diamond in top, only visible in preview
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 1*_NoSandbox }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		    }
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.05 }
	    }
	})
end

---- Ramparts...

function registerRampartPart(_nameFbxFile, _namePart, _nameAsset, _buildingPartType, _category)
    StoneGate:registerAssetId("models/" .. _nameFbxFile .. ".fbx/Prefab/smallSquaredTower/step_4_Sst_" .. _namePart .. "Part", StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration rampart part
    StoneGate:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_SST_" .. _nameAsset .. "_PREFAB",
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		}
	}
    })

    StoneGate:registerPrefabComponent("models/" .. _nameFbxFile .. ".fbx/Prefab/smallSquaredTower/step_4_Sst_" .. _namePart .. "Part", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = _buildingPartType
    })
end

registerRampartPart( "other/stoneDecorations", "StoneStairRailLeft", "STONE_STAIR_RAIL_LEFT", "MAJOR", "EXTENSION" )
registerRampartPart( "other/stoneDecorations", "StoneStairRailRight", "STONE_STAIR_RAIL_RIGHT", "MAJOR", "EXTENSION" )
registerRampartPart( "other/stoneDecorations", "Bridge1", "BRIDGE1", "MINOR", "EXTENSION" )
registerRampartPart( "other/stoneDecorations", "Bridge1P", "BRIDGE1P", "MINOR", "EXTENSION" )
registerRampartPart( "other/stoneDecorations", "HalfArchSmall", "HALF_ARCH_SMALL", "MINOR", "EXTENSION" )
registerRampartPart( "other/stoneDecorations", "HalfArchLarge", "HALF_ARCH_LARGE", "MINOR", "EXTENSION" )
registerRampartPart( "other/stoneDecorations", "StoneFloor", "STONE_FLOOR", "MAJOR", "EXTENSION" )
registerRampartPart( "other/stoneDecorations", "StoneFloorP", "STONE_FLOORP", "MAJOR", "EXTENSION" )

-- platforms
function configureStairPart(_nameFbxFile, _namePart, _nameAsset, _buildingPartType, _category)
    registerRampartPart(_nameFbxFile, _namePart, _nameAsset, _buildingPartType, _category)
    StoneGate:configurePrefabFlagList("models/" .. _nameFbxFile .. ".fbx/Prefab/smallSquaredTower/step_4_Sst_" .. _namePart .. "Part", { "PLATFORM" })
end
configureStairPart( "other/stoneDecorations", "Stair", "STAIR", "MAJOR", "EXTENSION" )
configureStairPart( "other/stoneDecorations", "StairP", "STAIRP", "MAJOR", "EXTENSION" )
configureStairPart( "other/stoneDecorations", "DoorStair", "DOOR_STAIR", "DOOR", "DOOR" )
configureStairPart( "other/stoneDecorations", "DoorStairP", "DOOR_STAIRP", "DOOR", "DOOR" )

function registerSmallSquaredStairTower(_nameFile, _namePart, _nameAsset, _option)
    -- Register prefab nodes
    StoneGate:registerAssetId("models/other/" .. _nameFile .. ".fbx/Prefab/smallSquaredTower/step_0_Sst_Root" .. _namePart .. _option .. "Part", StoneGateSetPrefix .. "_SST_ROOT_" .. _nameAsset .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/other/" .. _nameFile .. ".fbx/Prefab/smallSquaredTower/step_1_Sst_Tiling1" .. _namePart  .. _option .. "Part", StoneGateSetPrefix .. "_SST_TILING1_" .. _nameAsset .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/other/" .. _nameFile .. ".fbx/Prefab/smallSquaredTower/step_2_Sst_Tiling2" .. _namePart  .. _option .. "Part", StoneGateSetPrefix .. "_SST_TILING2_" .. _nameAsset .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/other/" .. _nameFile .. ".fbx/Prefab/smallSquaredTower/step_3_Sst_Top" .. _namePart  .. _option .. "Part", StoneGateSetPrefix .. "_SST_TOP_" .. _nameAsset .. _option .. "_PREFAB")
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_SST_TOP_" .. _nameAsset .. _option)
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_SST_TILING1_" .. _nameAsset .. _option)
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_SST_TILING2_" .. _nameAsset .. _option)

    -- Register the stone gate set small squared tower root part
    StoneGate:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SST_ROOT_" .. _nameAsset .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_SST_ROOT_" .. _nameAsset .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_SST_ROOT_" .. _nameAsset .. _option .. "_DESC",
	Category = "DOOR",
	BuildingZone = _SstStairBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_SST_ROOT_" .. _nameAsset .. _option .. "_PREFAB",
		EndPart = StoneGateSetPrefix .. "_SST_TOP_" .. _nameAsset .. _option .. "_PART",
		FillerList = {
			StoneGateSetPrefix .. "_SST_TILING2_" .. _nameAsset .. _option .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_SST_TILING1_" .. _nameAsset .. _option .. "_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0*_NoSandbox }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 1*_NoSandbox }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 0.5 }
	}
    })

    StoneGate:configurePrefabFlagList("models/other/" .. _nameFile .. ".fbx/Prefab/smallSquaredTower/step_3_Sst_Top" .. _namePart .. _option .. "Part", { "PLATFORM" })

    -- Register grass clearing components
    StoneGate:registerPrefabComponent("models/other/" .. _nameFile .. ".fbx/Prefab/smallSquaredTower/step_0_Sst_Root" .. _namePart .. _option .. "Part", {
	DataType = "COMP_DIRT_RECTANGLE",
	Size = { 4, 4 }
    })
end

registerSmallSquaredStairTower("stoneDecorations", "STAIR", "STAIR", "") -- Stair with stone ground
registerSmallSquaredStairTower("stoneDecorations", "STAIR", "STAIR", "P") -- Stair with wooden ground

function registerSmallSquaredStairEntranceTower(_nameFile, _namePart, _nameAsset, _option)
    registerSmallSquaredStairTower(_nameFile, _namePart, _nameAsset, _option)
    StoneGate:registerPrefabComponent("models/other/" .. _nameFile .. ".fbx/Prefab/smallSquaredTower/step_0_Sst_Root" .. _namePart .. _option .. "Part", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "DOOR"
    })
end

registerSmallSquaredStairEntranceTower("stoneDecorations", "DoorStair", "DOOR_STAIR", "") -- Plain stairs with stone ground
registerSmallSquaredStairEntranceTower("stoneDecorations", "DoorStair", "DOOR_STAIR", "P") -- Plain stairs with wooden ground

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix, _isVisibleWhenBuilt )
	StoneGate:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
        	Name = _nodePrefix .. "_NAME",
	        Description = _nodePrefix .. "_DESC",
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = 0*_NoSandbox }
		    }
	        },
                IsVisibleWhenBuilt = _isVisibleWhenBuilt  -- false to mask the green diamond and to add a comment in preview
	})
end

function registerRootPart(_nameAsset, _plankCost, _option, _OPTION, _buildingZone)
    -- Register common prefab nodes
    StoneGate:registerAssetId("models/other/woodenDecorations.fbx/Prefab/woodenFloor/step_0_WFDecoration_Root" .. _option .. "Part", StoneGateSetPrefix .. "_ROOT_" ..  _nameAsset .. _OPTION .. "_PREFAB")
    StoneGate:registerAssetId("models/other/woodenDecorations.fbx/Prefab/woodenFloor/step_1_WFDecoration_Tiling1" .. _option .. "Part", StoneGateSetPrefix .. "_TILING1_" ..  _nameAsset .. _OPTION .. "_PREFAB")
    StoneGate:registerAssetId("models/other/woodenDecorations.fbx/Prefab/woodenFloor/step_2_WFDecoration_Tiling2" .. _option .. "Part", StoneGateSetPrefix .. "_TILING2_" ..  _nameAsset .. _OPTION .. "_PREFAB")
    StoneGate:registerAssetId("models/other/woodenDecorations.fbx/Prefab/woodenFloor/step_3_WFDecoration_Top" .. _option .. "Part", StoneGateSetPrefix .. "_TOP_" ..  _nameAsset .. _OPTION .. "_PREFAB")
    -- Register common building parts
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_TOP_" ..  _nameAsset .. _OPTION, false) -- mask the green diamond
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_TILING1_" ..  _nameAsset .. _OPTION, true)
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_TILING2_" ..  _nameAsset .. _OPTION, true)

    -- Register the root of a scaleable wooden part
    StoneGate:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_ROOT_" ..  _nameAsset .. _OPTION .. "_PART",
	Name = StoneGateSetPrefix .. "_ROOT_" ..  _nameAsset .. _OPTION .. "_NAME",
	Description = StoneGateSetPrefix .. "_ROOT_" ..  _nameAsset .. _OPTION .. "_DESC",
	Category = "CORE",
        BuildingZone = _buildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_ROOT_" ..  _nameAsset .. _OPTION .. "_PREFAB",
		EndPart = StoneGateSetPrefix .. "_TOP_" ..  _nameAsset .. _OPTION .. "_PART",
		FillerList = {
			StoneGateSetPrefix .. "_TILING2_" ..  _nameAsset .. _OPTION .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_TILING1_" ..  _nameAsset .. _OPTION .. "_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = _plankCost*_NoSandbox }
		}
	},
        IsVisibleWhenBuilt = false  -- to mask the green diamond
    })
end

registerRootPart("PILLINGS", 1, "Platform", "_PLATFORM", _buildingZonePillings)
registerRootPart("PILLINGS", 2, "Stair",    "_STAIR",    _buildingZoneWoodenStair)
registerRootPart("PILLINGS", 2, "Door",     "_DOOR",     _buildingZoneWoodenStair)

function configurePlatformPart(_nameFbxFile, _nameModel, _namePart, _option)
    StoneGate:configurePrefabFlagList("models/other/" .. _nameFbxFile .. ".fbx/Prefab/" .. _nameModel .. "/step_3_".. _namePart .. "_Top" .. _option .. "Part", { "PLATFORM" })
end
configurePlatformPart("woodenDecorations", "woodenFloor", "WFDecoration", "Platform") -- not sure it is usefull
configurePlatformPart("woodenDecorations", "woodenFloor", "WFDecoration", "Stair")
configurePlatformPart("woodenDecorations", "woodenFloor", "WFDecoration", "Door")

StoneGate:registerPrefabComponent("models/other/woodenDecorations.fbx/Prefab/woodenFloor/step_0_WFDecoration_RootDoorPart", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "DOOR"
})

function registerSimpleDecorationPart(_nameFbxFile, _namePart, _nameAsset)
    StoneGate:registerAssetId("models/other/" .. _nameFbxFile .. ".fbx/Prefab/woodenFloor/" .. _namePart .. "Part", StoneGateSetPrefix .. "_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration roof part
    StoneGate:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_" .. _nameAsset .. "_PREFAB",
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = 2*_NoSandbox }
		}
	}
    })
end

function configurePlatformPart(_nameFbxFile, _namePart, _nameAsset)
    registerSimpleDecorationPart(_nameFbxFile, _namePart, _nameAsset)
    StoneGate:configurePrefabFlagList("models/other/" .. _nameFbxFile .. ".fbx/Prefab/woodenFloor/" .. _namePart .. "Part", { "PLATFORM" })
end
configurePlatformPart("woodenDecorations", "step_2_WFDecorationPlatform", "WFDECORATION_PLATFORM" ) -- platform

registerSimpleDecorationPart( "woodenDecorations", "WFDecorationPillings",             "WFDECORATION_PILLINGS")
registerSimpleDecorationPart( "woodenDecorations", "step_2_WFDecorationRail",          "WFDECORATION_RAIL" )
registerSimpleDecorationPart( "woodenDecorations", "step_2_WFDecorationDoubleSupport", "WFDECORATION_DOUBLE_SUPPORT" )
registerSimpleDecorationPart( "woodenDecorations", "step_2_WFDecorationAngleSupport",  "WFDECORATION_ANGLE_SUPPORT" )
registerSimpleDecorationPart( "woodenDecorations", "step_2_WFDecorationStairRailR",    "WFDECORATION_STAIR_RAIL_R" )
registerSimpleDecorationPart( "woodenDecorations", "step_2_WFDecorationStairRailL",    "WFDECORATION_STAIR_RAIL_L" )
